# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field, validator

class Validity(BaseModel):
    """
    Validity
    """
    minimum_periods: Optional[Any] = Field(..., alias="minimumPeriods", description="The minimum number of subscription periods that the plan must be renewed.")
    type: Optional[Any] = Field(..., description="The validity type of this plan.")
    unit: Optional[Any] = Field(..., description="The time unit of the subscription period.")
    value: Optional[Any] = Field(..., description="The amount of units (specified in the `unit` property) in each subscription period.")
    __properties = ["minimumPeriods", "type", "unit", "value"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('recurring', 'oneTime'):
            raise ValueError("must be one of enum values ('recurring', 'oneTime')")
        return value

    @validator('unit')
    def unit_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('day', 'month'):
            raise ValueError("must be one of enum values ('day', 'month')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Validity:
        """Create an instance of Validity from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if minimum_periods (nullable) is None
        # and __fields_set__ contains the field
        if self.minimum_periods is None and "minimum_periods" in self.__fields_set__:
            _dict['minimumPeriods'] = None

        # set to None if type (nullable) is None
        # and __fields_set__ contains the field
        if self.type is None and "type" in self.__fields_set__:
            _dict['type'] = None

        # set to None if unit (nullable) is None
        # and __fields_set__ contains the field
        if self.unit is None and "unit" in self.__fields_set__:
            _dict['unit'] = None

        # set to None if value (nullable) is None
        # and __fields_set__ contains the field
        if self.value is None and "value" in self.__fields_set__:
            _dict['value'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Validity:
        """Create an instance of Validity from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Validity.parse_obj(obj)

        _obj = Validity.parse_obj({
            "minimum_periods": obj.get("minimumPeriods"),
            "type": obj.get("type"),
            "unit": obj.get("unit"),
            "value": obj.get("value")
        })
        return _obj

